@@ -3,7 +3,7 @@
 // stringtypes.h              Created on: 16/4/99
 //                            Author    : David Riddoch (djr)
 //
-//    Copyright (C) 2003-2013 Apasphere Ltd
+//    Copyright (C) 2003-2015 Apasphere Ltd
 //    Copyright (C) 1996-1999 AT&T Laboratories Cambridge
 //
 //    This file is part of the omniORB library.
@@ -60,8 +60,10 @@ static inline char* dup(const char* s) {
   int   l = (int)strlen(s);
   char* r = alloc(l);
   if (r) {
-#ifdef _MSC_VER
+#if defined(_MSC_VER)
     strcpy_s(r, l+1, s);
+#elif defined(__darwin__)
+    strlcpy(r, s, l+1);
 #else
     strcpy(r, s);
 #endif
